@@ -87,6 +87,7 @@ class NeuronDevice(TargetsNeurons, TargetsSections, SimulationComponent):
     device_types = [
         "spike_generator",
         "current_clamp",
+        "voltage_clamp",
         "spike_recorder",
         "voltage_recorder",
         "synapse_recorder",
@@ -334,6 +335,7 @@ class NeuronAdapter(SimulatorAdapter):
         self.result = SimulationResult()
         if self.pc_id == 0:
             # Record the time
+            self.h.time
             self.result.create_recorder(
                 lambda: tuple(["time"]),
                 lambda: np.array(self.h.time),
@@ -369,7 +371,7 @@ class NeuronAdapter(SimulatorAdapter):
                 break
         report("Finished simulation.", level=2)
 
-    def collect_output(self):
+    def collect_output(self, simulator):
         import h5py, time
 
         timestamp = str(time.time()).split(".")[0] + str(random.random()).split(".")[1]
@@ -558,6 +560,7 @@ class NeuronAdapter(SimulatorAdapter):
             # Re-initialise the device
             # TODO: Switch to better config in v4
             device.initialise(device.scaffold)
+            device.validate_specifics()
             device.initialise_targets()
             device.initialise_patterns()
 
@@ -579,6 +582,21 @@ class NeuronAdapter(SimulatorAdapter):
         terminal_relays = {}
         intermediate_relays = {}
         output_handler = self.scaffold.output_formatter
+        cell_types = self.scaffold.get_cell_types()
+        type_lookup = {
+            ct.name: range(min(ids), max(ids) + 1)
+            for ct, ids in zip(
+                cell_types, (ct.get_placement_set().identifiers for ct in cell_types)
+            )
+        }
+
+        def lookup(i):
+            for n, t in type_lookup.items():
+                if i in t:
+                    return n
+            else:
+                return None
+
         for connection_model in self.connection_models.values():
             name = connection_model.name
             # Get the connectivity set associated with this connection model
@@ -624,43 +642,36 @@ class NeuronAdapter(SimulatorAdapter):
 
         report("Relays indexed, resolving intermediates.")
 
+        interm_transfer = {k: [] for k in intermediate_relays.keys()}
         while len(intermediate_relays) > 0:
             intermediates_to_remove = []
             for intermediate, targets in intermediate_relays.items():
                 for target in targets:
                     if target in intermediate_relays:
-                        # This target of this intermediary is also an intermediary and
-                        # cannot be resolved to a terminal at this point, so we wait until
-                        # a next iteration where the intermediary target might have been
-                        # resolved.
+                        # This target of this intermediary is also an
+                        # intermediary and cannot be resolved to a terminal at
+                        # this point, so we wait until a next iteration where
+                        # the intermediary target might have been resolved.
                         continue
-                    if target in terminal_relays:
-                        # The target is a terminal relay and can be removed from our
-                        # intermediary target list and its terminal targets added to our
-                        # terminal target list.
-                        try:
-                            arr = terminal_relays[intermediate]
-                        except:
-                            arr = []
-                            terminal_relays[intermediate] = arr
+                    elif target in terminal_relays:
+                        # The target is a terminal relay and can be removed from
+                        # our intermediary target list and its terminal targets
+                        # added to our terminal target list.
+                        arr = interm_transfer[intermediate]
+                        assert all(
+                            isinstance(t, tuple) for t in terminal_relays[target]
+                        ), f"Terminal relay {lookup(target)} {target} contains non-terminal targets: {terminal_relays[target]}"
                         arr.extend(terminal_relays[target])
                         targets.remove(target)
-                        # If we now have no more intermediary  targets we can be removed
-                        # from the intermediary relay list.
-                        if len(targets) == 0:
-                            intermediates_to_remove.append(intermediate)
                     else:
-                        # The target is not a relay at all and can be added to our
-                        # terminal target list
-                        try:
-                            arr = terminal_relays[intermediate]
-                        except:
-                            arr = []
-                            terminal_relays[intermediate] = arr
-                        arr.append(target)
-                        targets.remove(target)
-                        if len(targets) == 0:
-                            intermediates_to_remove.append(intermediate)
+                        raise RelayError(
+                            f"Non-relay {lookup(target)} {target} found in intermediate relay map."
+                        )
+                # If we have no more intermediary targets we can be removed from
+                # the intermediary relay list and be moved to the terminals.
+                if not targets:
+                    intermediates_to_remove.append(intermediate)
+                    terminal_relays[intermediate] = interm_transfer.pop(intermediate)
             for intermediate in intermediates_to_remove:
                 report(
                     "Intermediate resolved to",
@@ -675,7 +686,10 @@ class NeuronAdapter(SimulatorAdapter):
         # Filter out all relays to targets not on this node.
         self.relay_scheme = {}
         for relay, targets in terminal_relays.items():
-            node_targets = list(filter(lambda x: int(x[0]) in self.node_cells, targets))
+            assert all(
+                isinstance(t, tuple) for t in terminal_relays[target]
+            ), f"Terminal relay {lookup(target)} {target} contains non-terminal targets: {terminal_relays[target]}"
+            node_targets = [x for x in targets if int(x[0]) in self.node_cells]
             self.relay_scheme[relay] = node_targets
         report(
             "Node",
@@ -723,6 +737,7 @@ class LocationRecorder(SimulationRecorder):
         self.id = cell.ref_id
         self.tag = str(cell.ref_id)
         if section is not None:
+            meta["section"] = gc.sections.index(section)
             self.tag += "." + section.name().split(".")[-1]
             if x is not None:
                 self.tag += "(" + str(x) + ")"
@@ -732,9 +747,9 @@ class LocationRecorder(SimulationRecorder):
 
     def get_data(self):
         if self.time_recorder:
-            return np.hstack((np.array(self.recorder), np.array(self.time_recorder)))
+            return np.column_stack((list(self.recorder), list(self.time_recorder)))
         else:
-            return np.array(self.recorder)
+            return np.array(list(self.recorder))
 
     def get_meta(self):
         return self.meta
@@ -752,5 +767,5 @@ class TargetLocation:
 
 class SpikeRecorder(LocationRecorder):
     def get_data(self):
-        recording = np.array(self.recorder)
-        return np.vstack((np.ones(recording.shape) * self.id, recording))
+        recording = np.array(list(self.recorder))
+        return np.column_stack((np.ones(recording.shape) * self.id, recording))
