@@ -1,6 +1,7 @@
 from ..adapter import NeuronDevice
 from ....simulation.results import SimulationRecorder, PresetPathMixin, PresetMetaMixin
 from ....helpers import listify_input
+from ....exceptions import *
 from ....functions import poisson_train
 from ....reporting import report, warn
 import numpy as np
@@ -8,6 +9,12 @@ import numpy as np
 
 class SpikeGenerator(NeuronDevice):
     defaults = {"record": True}
+    casts = {
+        "radius": float,
+        "origin": [float],
+        "synapses": [str],
+    }
+    required = ["targetting", "device", "io", "synapses"]
 
     def implement(self, target, location):
         cell = location.cell
@@ -35,30 +42,44 @@ class SpikeGenerator(NeuronDevice):
                 )
 
     def validate_specifics(self):
-        if "weight" not in self.parameters:
-            self.parameters["weight"] = 1
+        if not hasattr(self, "spike_times") and not hasattr(self, "parameters"):
+            raise ConfigurationError(
+                f"{self.name} is missing `spike_times` or `parameters`"
+            )
         self.synapses = listify_input(self.synapses)
 
     def create_patterns(self):
         report("Creating spike generator patterns for '{}'".format(self.name), level=3)
-        patterns = {}
+        targets = self.get_targets()
         if hasattr(self, "spike_times"):
-            return {target: self.spike_times for target in self.get_targets()}
-        interval = float(self.parameters["interval"])
-        number = int(self.parameters["number"])
-        start = float(self.parameters["start"])
-        noise = "noise" in self.parameters and self.parameters["noise"]
-        if not noise:
-            pattern = [start + i * interval for i in range(number)]
-        for target in self.get_targets():
+            pattern = self.spike_times
+            if self.record:
+                for target in targets:
+                    self.adapter.result.add(GeneratorRecorder(self, target, pattern))
+            patterns = {target: pattern for target in targets}
+        else:
+            interval = float(self.parameters["interval"])
+            number = int(self.parameters["number"])
+            start = float(self.parameters["start"])
+            noise = "noise" in self.parameters and self.parameters["noise"]
             frequency = 1.0 / interval
             duration = interval * number
-            if noise:
-                pattern = list(poisson_train(frequency, duration, start))
-            patterns[target] = pattern
+            if not noise:
+                # Create only 1 copy of the pattern array, might be surprising
+                # for tinkering users, but in the framework the created patterns
+                # should be used read only in `get_pattern(target)` to pass as
+                # input to a VecStim.
+                pattern = [start + i * interval for i in range(number)]
+                patterns = {target: pattern for target in targets}
+            else:
+                patterns = {
+                    target: list(poisson_train(frequency, duration, start))
+                    for target in targets
+                }
             if self.record:
-                self.adapter.result.add(GeneratorRecorder(self, target, pattern))
-            report("Pattern {} for {}.".format(pattern, target), level=4)
+                for target, pattern in patterns.items():
+                    self.adapter.result.add(GeneratorRecorder(self, target, pattern))
+                    report("Pattern {} for {}.".format(pattern, target), level=4)
         return patterns
 
     def get_pattern(self, target, cell=None, section=None, synapse=None):
